@@ -509,14 +509,17 @@ switch_mode (GdmGreeterLoginWindow *login_window,
         switch (number) {
         case MODE_SELECTION:
                 set_log_in_button_mode (login_window, LOGIN_BUTTON_HIDDEN);
+                set_sensitive (login_window, TRUE);
                 gtk_widget_hide (login_window->priv->session_option_widget);
                 break;
         case MODE_TIMED_LOGIN:
                 set_log_in_button_mode (login_window, LOGIN_BUTTON_TIMED_LOGIN);
+                set_sensitive (login_window, TRUE);
                 gtk_widget_show (login_window->priv->session_option_widget);
                 break;
         case MODE_AUTHENTICATION:
                 set_log_in_button_mode (login_window, LOGIN_BUTTON_ANSWER_QUERY);
+                set_sensitive (login_window, FALSE);
                 gtk_widget_show (login_window->priv->session_option_widget);
                 break;
         default:
@@ -682,7 +685,6 @@ reset_dialog (GdmGreeterLoginWindow *login_window,
                 switch_mode (login_window, dialog_mode);
         }
 
-        set_sensitive (login_window, TRUE);
         set_ready (login_window);
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
         update_banner_message (login_window);
@@ -762,6 +764,7 @@ gdm_greeter_login_window_info (GdmGreeterLoginWindow *login_window,
         g_debug ("GdmGreeterLoginWindow: info: %s", text);
 
         set_message (GDM_GREETER_LOGIN_WINDOW (login_window), text);
+        maybe_show_cancel_button (login_window);
 
         return TRUE;
 }
@@ -773,6 +776,7 @@ gdm_greeter_login_window_problem (GdmGreeterLoginWindow *login_window,
         g_return_val_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window), FALSE);
 
         g_debug ("GdmGreeterLoginWindow: problem: %s", text);
+        maybe_show_cancel_button (login_window);
 
         set_message (GDM_GREETER_LOGIN_WINDOW (login_window), text);
         gdk_window_beep (gtk_widget_get_window (GTK_WIDGET (login_window)));
@@ -1269,6 +1273,18 @@ create_computer_info (GdmGreeterLoginWindow *login_window)
 #define INVISIBLE_CHAR_NONE          0
 
 
+static void
+register_custom_types (GdmGreeterLoginWindow *login_window)
+{
+        GType types[] = { GDM_TYPE_USER_CHOOSER_WIDGET,
+                          GDM_TYPE_SESSION_OPTION_WIDGET };
+        int i;
+
+        for (i = 0; i < G_N_ELEMENTS (types); i++) {
+                g_debug ("Registering type '%s'", g_type_name (types[i]));
+        }
+}
+
 static void
 load_theme (GdmGreeterLoginWindow *login_window)
 {
@@ -1280,6 +1296,8 @@ load_theme (GdmGreeterLoginWindow *login_window)
 
         gdm_profile_start (NULL);
 
+        register_custom_types (login_window);
+
         login_window->priv->builder = gtk_builder_new ();
         if (!gtk_builder_add_from_file (login_window->priv->builder, UIDIR "/" UI_XML_FILE, &error)) {
                 g_warning ("Couldn't load builder file: %s", error->message);
@@ -1313,16 +1331,10 @@ load_theme (GdmGreeterLoginWindow *login_window)
         box = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, "window-frame"));
         gtk_container_add (GTK_CONTAINER (login_window), box);
 
-        /* FIXME: user and session chooser should get loaded from ui file instead
-         */
-        login_window->priv->user_chooser = gdm_user_chooser_widget_new ();
-        box = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, "selection-box"));
-        gtk_box_pack_start (GTK_BOX (box), login_window->priv->user_chooser, TRUE, TRUE, 0);
-        gtk_box_reorder_child (GTK_BOX (box), login_window->priv->user_chooser, 0);
+        login_window->priv->user_chooser = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, "user-chooser"));
 
         gdm_user_chooser_widget_set_show_only_chosen (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
 
-
         g_signal_connect (login_window->priv->user_chooser,
                           "loaded",
                           G_CALLBACK (on_users_loaded),
@@ -1341,16 +1353,7 @@ load_theme (GdmGreeterLoginWindow *login_window)
                                  G_CALLBACK (on_user_chooser_visibility_changed),
                                  login_window);
 
-        login_window->priv->session_option_widget = gdm_session_option_widget_new ();
-        box = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, "buttonbox"));
-        g_object_set (G_OBJECT (login_window->priv->session_option_widget),
-                      "xscale", 0.0,
-                      "yscale", 0.0,
-                      "xalign", 0.0,
-                      "yalign", 1.0,
-                      NULL);
-        gtk_container_add (GTK_CONTAINER (box), login_window->priv->session_option_widget);
-        gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (box), login_window->priv->session_option_widget, TRUE);
+        login_window->priv->session_option_widget = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, "session-option-widget"));
 
         g_signal_connect (login_window->priv->session_option_widget,
                           "activated",
